#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <math.h>

#include <hpic.h>

int main(int argc, char *argv[])
{
  char ch = '-';
  size_t i, j, k;

  char infile[HPIC_STRNL];
  char outfile[HPIC_STRNL];
  double upper = 0.0;
  double lower = 0.0;
  double outval = 0.0;
  int setall = 0;
  int doupper = 0;
  int dolower = 0;
  size_t mapnum = 0;

  size_t nside;
  int order;
  int coord;
  int keepup;
  int keeplow;
  int type;
  size_t nmaps;
  size_t npix;
  char command[HPIC_STRNL];
  char creator[HPIC_STRNL];
  char extname[HPIC_STRNL];
  char comment[HPIC_STRNL];

  hpic_int *pix = NULL;
  hpic_fltarr *maps = NULL;
  hpic_int *hits = NULL;
  hpic_float *errs = NULL;
  hpic_keys *keys;
  hpic_float *tempmap;

  int temp;
  float val;

  if (argc == 1) {
    printf("\n");
    printf("<******************************************************>\n");
    printf("Usage:  hpic_thresh -in <input FITS file>\n");
    printf("        -out <output FITS file>\n");
    printf("        -mapnum <map number to process>\n");
    printf("        -upper <upper threshold>\n");
    printf("        -lower <lower threshold>\n");
    printf
      ("        -outval <optionally set all in-range pixels to this value>\n");
    printf("<******************************************************>\n\n");
    exit(1);
  }

  /* parse command line */

  for (i = 0; i < argc; i++) {
    if (argv[i][0] == ch) {
      if (!strcmp(argv[i], "-in")) {
        strncpy(infile, argv[i + 1], HPIC_STRNL);
        i++;
      }
      if (!strcmp(argv[i], "-out")) {
        strncpy(outfile, argv[i + 1], HPIC_STRNL);
        i++;
      }
      if (!strcmp(argv[i], "-mapnum")) {
        mapnum = (size_t) atoi(argv[i + 1]);
        i++;
      }
      if (!strcmp(argv[i], "-upper")) {
        doupper = 1;
        upper = atof(argv[i + 1]);
        i++;
      }
      if (!strcmp(argv[i], "-lower")) {
        dolower = 1;
        lower = atof(argv[i + 1]);
        i++;
      }
      if (!strcmp(argv[i], "-outval")) {
        outval = atof(argv[i + 1]);
        setall = 1;
        i++;
      }
    }
  }

  /* process FITS file */

  strcpy(comment, "");

  if (hpic_fits_map_test(infile, &nside, &order, &coord, &type, &nmaps)) {

    if (mapnum >= nmaps) {
      fprintf(stderr,
              "hpic_thresh:  file contains %lu maps (columns), cannot access column %lu\n",
              nmaps, mapnum);
      exit(1);
    }

    keys = hpic_keys_alloc();
    maps = hpic_fltarr_alloc(nmaps);
    for (i = 0; i < nmaps; i++) {
      tempmap = hpic_float_alloc(nside, order, coord, HPIC_AUTO);
      hpic_fltarr_set(maps, i, tempmap);
    }

    if (type == HPIC_FITS_FULL) {
      hpic_fits_full_read(infile, creator, extname, maps, keys);
      tempmap = hpic_fltarr_get(maps, mapnum);
      for (i = 0; i < hpic_nside2npix(nside); i++) {
        val = hpic_float_get(tempmap, i);
        if (!hpic_is_fnull(val)) {
          if (doupper) {
            if (val <= (float)upper) {
              keepup = 1;
            } else {
              keepup = 0;
            }
          } else {
            keepup = 1;
          }
          if (dolower) {
            if (val >= (float)lower) {
              keeplow = 1;
            } else {
              keeplow = 0;
            }
          } else {
            keeplow = 1;
          }
          if (keepup && keeplow) {
            if (setall) {
              hpic_float_set(tempmap, i, (float)outval);
            }
          } else {
            hpic_float_set(tempmap, i, HPIC_NULL);
          }
        }
      }

      hpic_fits_full_write(outfile, "hpic_thresh", "Threshold Map", comment,
                           maps, keys);
    } else {
      errs = hpic_float_alloc(nside, order, coord, HPIC_AUTO);
      hits = hpic_int_alloc(nside, order, coord, HPIC_AUTO);
      pix = hpic_int_alloc(nside, order, coord, HPIC_AUTO);

      hpic_fits_cut_read(infile, creator, extname, pix, hits, errs, maps,
                         keys);
      tempmap = hpic_fltarr_get(maps, mapnum);
      for (i = 0; i < hpic_nside2npix(nside); i++) {
        val = hpic_float_get(tempmap, i);
        if (!hpic_is_fnull(val)) {
          if (doupper) {
            if (val <= (float)upper) {
              keepup = 1;
            } else {
              keepup = 0;
            }
          } else {
            keepup = 1;
          }
          if (dolower) {
            if (val >= (float)lower) {
              keeplow = 1;
            } else {
              keeplow = 0;
            }
          } else {
            keeplow = 1;
          }
          if (keepup && keeplow) {
            if (setall) {
              hpic_float_set(tempmap, i, (float)outval);
            }
          } else {
            hpic_float_set(tempmap, i, HPIC_NULL);
          }
        }
      }

      hpic_fits_cut_write(outfile, "hpic_thresh", "Threshold Map", comment,
                          pix, hits, errs, maps, keys);

      hpic_int_free(pix);
      hpic_int_free(hits);
      hpic_float_free(errs);
    }

    hpic_keys_free(keys);
    for (i = 0; i < nmaps; i++) {
      tempmap = hpic_fltarr_get(maps, i);
      hpic_float_free(tempmap);
    }
    hpic_fltarr_free(maps);

  } else {
    fprintf(stderr, "hpic_thresh:  FITS file is not a recognized type\n");
    exit(1);
  }

  return 0;
}
